<?php
/**
 * Plugin Name: Fortis for WooCommerce
 * Description: Receive payments using the Fortis payments provider.
 * Author: Fortis Payment Systems
 * Author URI: https://fortispay.com/
 * Version: 1.0.6
 * Requires at least: 6.0
 * Tested up to: 6.4.3
 * WC tested up to: 8.7.0
 * WC requires at least: 7.0
 *
 * @package Fortis for WooCommerce
 *
 * Developer: App Inlet (Pty) Ltd
 * Developer URI: https://www.appinlet.com/
 *
 * Copyright: 2024 Fortis Payment Systems, LLC (“Fortis”)
 * License: GNU General Public License v3.0
 * License URI: http://www.gnu.org/licenses/gpl-3.0.html
 * Text Domain: fortis-for-woocommerce
 */

use Automattic\WooCommerce\Utilities\FeaturesUtil;
use JetBrains\PhpStorm\NoReturn;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

add_action( 'plugins_loaded', 'fortis_init', 0 );

const WC_GATEWAY_FORTIS = 'classes/WC_Gateway_Fortis.php';

/**
 * Initialize the gateway.
 *
 * @since 1.0.0
 * @noinspection PhpUnused
 */
function fortis_init() {
	if ( ! class_exists( 'WC_Payment_Gateway' ) ) {
		return;
	}

	require_once plugin_basename( WC_GATEWAY_FORTIS );

	add_filter( 'woocommerce_payment_gateways', 'add_fortis_gateway' );

	/**
	 * Custom order action - query order status
	 * Add custom action to order actions select box
	 */
	add_action(
		'woocommerce_order_actions',
		array( WC_Gateway_Fortis_Admin_Actions::class, 'fortis_add_order_meta_box_action' )
	);

	add_action( 'woocommerce_before_cart', array( WC_Gateway_Fortis::class, 'show_cart_messages' ), 10, 1 );
} // End fortis_init()

add_filter( 'woocommerce_available_payment_gateways', 'payment_gateway_disable_private' );
add_filter( 'woocommerce_order_needs_payment', array( WC_Gateway_Fortis::class, 'order_needs_payment' ), 10, 3 );

add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'fortis_settings_link' );

/**
 * Makes json request
 *
 * @return void
 */
#[NoReturn] function fortis_ajax_request(): void {
	require_once plugin_dir_path( __FILE__ ) . 'includes/blocks/class-wc-fortis-payments-blocks.php';

	$wc_blocks = new WC_Gateway_Fortis_Blocks_Support();
	$wc_blocks->initialize();

	echo wp_json_encode( $wc_blocks->get_payment_method_data() );
	die();
}

add_action( 'wp_ajax_fortis_ajax_request', 'fortis_ajax_request' );
add_action( 'wp_ajax_nopriv_fortis_ajax_request', 'fortis_ajax_request' );

/**
 * Generate iframe form
 *
 * @return void
 */
#[NoReturn] function generate_iframe_form(): void {
	$fortis_gateway = new WC_Gateway_Fortis();

	$html = $fortis_gateway->generate_fortis_form(); // This has been escaped with wp_kses.

	echo $html; // This has been escaped with wp_kses.
	die();
}

add_action( 'wp_ajax_generate_iframe_form', 'generate_iframe_form' );
add_action( 'wp_ajax_nopriv_generate_iframe_form', 'generate_iframe_form' );
add_action(
	'woocommerce_admin_order_data_after_payment_info',
	array(
		WC_Gateway_Fortis::class,
		'add_ach_refund_message',
	),
	10,
	1
);

/**
 * Links
 *
 * @param array $links Links.
 *
 * @return array
 */
function fortis_settings_link( array $links ): array {
	$url           = get_admin_url() . 'admin.php?page=wc-settings&tab=checkout&section=fortis';
	$settings_link = '<a href="' . esc_url( $url ) . '">' .
					esc_html_e( 'Settings', 'fortis-for-woocommerce' ) . '</a>';
	$links[]       = $settings_link;

	return $links;
}

/**
 * Add the gateway to WooCommerce
 *
 * @param array $methods Methods.
 *
 * @return array
 * @since 1.0.0
 */
function add_fortis_gateway( array $methods ): array {
	require_once WC_GATEWAY_FORTIS;

	$methods[] = 'WC_Gateway_Fortis';

	return $methods;
} // End add_fortis_gateway().

/**
 * Disable the gateway
 *
 * @param array $available_gateways Gateways available.
 *
 * @return array
 */
function payment_gateway_disable_private( array $available_gateways ): array {
	require_once WC_GATEWAY_FORTIS;

	$fortis_api = new FortisApi( 'Fortis' );

	if ( '' === $fortis_api->framework->get_user_id() ||
		'' === $fortis_api->framework->get_user_api_key() ||
		'yes' !== $fortis_api->framework->get_enabled() ) {
		unset( $available_gateways['fortis'] );
	}

	return $available_gateways;
}

/**
 * Registers WooCommerce Blocks integration.
 */
class WC_Fortis_Payments {


	/**
	 * Plugin bootstrapping.
	 */
	public static function init() {
		// Registers WooCommerce Blocks integration.
		add_action(
			'woocommerce_blocks_loaded',
			array( __CLASS__, 'fortis_gateway_fortis_woocommerce_block_support' )
		);
	}

	/**
	 * Add the Fortis Payment gateway to the list of available gateways.
	 *
	 * @param array $gateways Gateways.
	 */
	public static function add_gateway( array $gateways ): array {
		$gateways[] = 'WC_Gateway_Fortis';

		return $gateways;
	}

	/**
	 * Plugin includes.
	 */
	public static function includes() {
		// Make the WC_Gateway_Fortis class available.
		if ( class_exists( 'WC_Payment_Gateway' ) ) {
			require_once 'classes/WC_Gateway_Fortis.php';
		}
	}

	/**
	 * Plugin url.
	 *
	 * @return string
	 */
	public static function plugin_url() {
		return untrailingslashit( plugins_url( '/', __FILE__ ) );
	}

	/**
	 * Plugin url.
	 *
	 * @return string
	 */
	public static function plugin_abspath() {
		return trailingslashit( plugin_dir_path( __FILE__ ) );
	}

	/**
	 * Registers WooCommerce Blocks integration.
	 */
	public static function fortis_gateway_fortis_woocommerce_block_support() {
		if ( class_exists( 'Automattic\WooCommerce\Blocks\Payments\Integrations\AbstractPaymentMethodType' ) ) {
			require_once plugin_dir_path( __FILE__ ) . 'includes/blocks/class-wc-fortis-payments-blocks.php';
			add_action(
				'woocommerce_blocks_payment_method_type_registration',
				function ( Automattic\WooCommerce\Blocks\Payments\PaymentMethodRegistry $payment_method_registry ) {
					$payment_method_registry->register( new WC_Gateway_Fortis_Blocks_Support() );
				}
			);
		}
	}
}

/**
 * Declares support for HPOS.
 *
 * @return void
 */
function fortis_declare_hpos_compatibility() {
	if ( class_exists( '\Automattic\WooCommerce\Utilities\FeaturesUtil' ) ) {
		FeaturesUtil::declare_compatibility(
			'custom_order_tables',
			__FILE__,
			true
		);
	}
}

add_action( 'before_woocommerce_init', 'fortis_declare_hpos_compatibility' );


WC_Fortis_Payments::init();
